@@ -6,29 +6,21 @@ import pandas as pd
 import panel as pn
 import param
 
+import sim2bids.generate.subjects as subj
 from sim2bids import utils
-from sim2bids.app import app, utils as app_utils
-from sim2bids.templates import user_guide as ug, templates
+from sim2bids.app import app
+from sim2bids.templates import user_guide as ug
 from sim2bids.validate import validate
 
-
 JE_FIELDS = ['Units', 'Description', 'CoordsRows', 'CoordsColumns', 'ModelEq', 'ModelParam', 'SourceCode',
              'SourceCodeVersion', 'SoftwareVersion', 'SoftwareName', 'SoftwareRepository', 'Network']
 UNITS = ['s', 'm', 'ms', 'degrees', 'radians']
-OPTIONS = ['App 101', 'Preprocess data', 'BEP034']
+OPTIONS = ['App 101', 'Preprocess data', 'Supported files', 'Functionality', 'BEP034']
+
 REQUIRED = None
+
 AUTOFILL = True
 
-rhythms = pn.Column('#### Specify rhythms', pn.widgets.CheckBoxGroup(
-    value=[], options=['Alpha', 'Beta', 'Delta', 'Gamma', 'Theta'],
-    inline=True, margin=(-10, 0, 0, 0)))
-
-speed = pn.Column('#### Specify `conduction speed` folder name',
-                  pn.widgets.TextInput(value='cspeed', margin=(-10, 0, 0, 0)))
-
-global_coupling = pn.Column('#### Specify `global coupling scaling factor` folder name',
-                            pn.widgets.TextInput(value='csf', margin=(-10, 0, 0, 0)))
-
 
 class MainArea(param.Parameterized):
     # generate files button
@@ -42,10 +34,7 @@ class MainArea(param.Parameterized):
 
     # sidebar components
     output_path = pn.widgets.TextInput(value=app.OUTPUT, margin=(-20, 10, 0, 10))
-    # app.OUTPUT = output_path.value
-
-    input_path = pn.widgets.TextInput(value=app.INPUT, margin=(-20, 10, 0, 10))
-    # app.OUTPUT = output_path.value
+    app.OUTPUT = output_path.value
 
     desc = pn.widgets.TextInput(value='default', max_length=30, margin=(-20, 10, 0, 10))
     app.DESC = desc.value
@@ -54,13 +43,6 @@ class MainArea(param.Parameterized):
     checkbox_group = pn.widgets.CheckBoxGroup(value=['Traverse subfolders', 'Autocomplete columns'],
                                               options=checkbox_options,
                                               margin=(-20, 10, 0, 10))
-
-    speed = pn.Column('#### Specify `conduction speed` folder name',
-                      pn.widgets.TextInput(value=app.COND_SPEED, margin=(-10, 0, 0, 0)))
-
-    global_coupling = pn.Column('#### Specify `global coupling scaling factor` folder name',
-                                pn.widgets.TextInput(value=app.GLB_COUP_SF, margin=(-10, 0, 0, 0)))
-
     rename_files = pn.WidgetBox()
 
     def __init__(self, **params):
@@ -71,7 +53,6 @@ class MainArea(param.Parameterized):
         self.subjects = None
         self.length = 0
         self.struct = ''
-        self.to_rename = []
 
     @pn.depends('text_input.value', watch=True)
     def _select_path(self):
@@ -88,52 +69,51 @@ class MainArea(param.Parameterized):
     @pn.depends('cross_select.value', watch=True)
     def _generate_path(self):
         self.structure.value = ''
-        selected = self.cross_select.value
 
-        if len(selected) == 0:
+        if len(self.cross_select.value) == 0:
             utils.reset_values()
             app.CODE = None
-            self.to_rename = []
-            self.to_rename_path = []
-            self.structure.value = []
-            self.rename_files = pn.WidgetBox()
-            self.to_rename = None
-            self.to_rename_path = None
 
-        if len(selected) > 0:
-            # check files for preprocessing step
+            for _ in self.rename_files:
+                self.rename_files.pop(-1)
+
+        if self.length != len(self.cross_select.value):
             utils.reset_values()
-            path = self.text_input.value
-            app.preprocess_input(path, selected)
-            self.to_rename = validate.filter(app_utils.get_content(path, selected, basename=True))
-            print(self.to_rename)
-            self.to_rename_path = validate.filter(app_utils.get_content(path, selected), self.to_rename)
+            app.CODE = None
 
-            if len(self.to_rename) > len(self.rename_files) + 1:
-                self.rename_files += [*utils.append_widgets(self.to_rename)]
-                self.rename_files.append(pn.Param(self, parameters=['rename_btn'], show_name=False,
-                                                  widgets={'rename_btn': {'button_type': 'primary'}}))
-            else:
-                self.rename_files = pn.WidgetBox()
+            for _ in self.rename_files:
+                self.rename_files.pop(-1)
 
-            self.length = len(selected)
+        if len(self.cross_select.value) > 0:
+            # Step 1: traverse files and check for problems
+            # appert.check_input(path=self.text_input.value, files=self.cross_select.value)
+            self.subjects, self.struct = app.main(path=self.text_input.value,
+                                                  files=self.cross_select.value,
+                                                  save=False, layout=False)
+
+            self.length = len(self.cross_select.value)
 
     def _generate_files(self, event=None):
-        if validate.IS_RENAMED:
-            _ = app.main(path=self.text_input.value,
-                         files=validate.RENAMED,
-                         # files=utils.get_all_files(validate.RENAMED, self.cross_select.value, self.text_input.value),
-                         subs=self.subjects, save=True, layout=True)
-        else:
-            _ = app.main(path=self.text_input.value, files=self.cross_select.value,
-                              subs=self.subjects, save=True, layout=True)
+        _ = app.main(path=self.text_input.value, files=self.cross_select.value,
+                     subs=self.subjects, save=True, layout=True)
+        subj.TO_RENAME = None
         app.ALL_FILES = None
-        utils.reset_values()
 
     def _generate_struct(self, event=None):
         self.subjects, self.struct = app.main(path=self.text_input.value,
                                               files=self.cross_select.value,
                                               save=False, layout=True)
+
+        if subj.TO_RENAME is not None:
+            if len(subj.TO_RENAME) > len(self.rename_files):
+                self.rename_files += [*utils.append_widgets(subj.TO_RENAME)]
+                self.rename_files.append(pn.Param(self, parameters=['rename_btn'],
+                                                  show_name=False,
+                                                  widgets={'rename_btn': {'button_type': 'primary'}}))
+        else:
+            for _ in self.rename_files:
+                self.rename_files.pop(-1)
+
         self.structure.value = self.struct
 
     @pn.depends('checkbox_group.value', watch=True)
@@ -146,22 +126,9 @@ class MainArea(param.Parameterized):
         # whether to autofill all files
         AUTOFILL = True if self.checkbox_options[1] in self.checkbox_group.value else False
 
-        # TODO: add copy folder functionality
         # whether to not alter original input folder
-        # if self.checkbox_options[2] in self.checkbox_group.value:
-        #     self.text_input.value = app.duplicate_folder(self.text_input.value)
-
-    @pn.depends('input_path.value', watch=True)
-    def _store_input(self):
-        inputs = self.input_path.value
-
-        if len(inputs) > 0:
-            if not os.path.exists(inputs):
-                pn.state.notifications.success(f'Folder {inputs} is created...')
-                os.mkdir(inputs)
-
-            pn.state.notifications.success(f'Folder `{inputs}` is selected as input folder...')
-            app.INPUT = inputs
+        if self.checkbox_options[2] in self.checkbox_group.value:
+            self.text_input.value = app.duplicate_folder(self.text_input.value)
 
     @pn.depends('output_path.value', watch=True)
     def _store_output(self):
@@ -176,33 +143,27 @@ class MainArea(param.Parameterized):
             app.OUTPUT = output
 
     def _rename(self, event=None):
-        validate.validate(self.rename_files, self.to_rename_path, self.text_input.value, self.cross_select.value)
-        self.rename_files = pn.WidgetBox()
-        self.to_rename = None
-        self.to_rename_path = None
+        validate.validate(self.rename_files, app.ALL_FILES)
 
     @pn.depends('desc.value', watch=True)
     def _change_desc(self):
         app.DESC = self.desc.value
-        pn.state.notifications.success(f'Description {app.DESC} has been saved...')
 
     def view(self):
         main = pn.Tabs(
             ('Select Files', pn.Column(pn.pane.Markdown(GET_STARTED),
                                        self.text_input,
                                        self.cross_select,
-                                       pn.Param(self, parameters=['gen_btn'],
-                                                show_name=False, widgets={'gen_btn': {'button_type': 'primary'}}),
-                                       # pn.Row(
-                                       #     pn.Param(self, parameters=['gen_struct'],
-                                       #              show_name=False,
-                                       #              widgets={'gen_struct': {'button_type': 'primary'}}),
-                                       #     pn.Param(self, parameters=['gen_btn'],
-                                       #              show_name=False, widgets={'gen_btn': {'button_type': 'primary'}}),
-                                       #     sizing_mode='stretch_width', margin=(50, 0, 0, 0)
-                                       # ),
-                                       )),
-            ('Preprocess Data', pn.Column(PREPROCESS, self.rename_files)),
+                                       pn.Row(
+                                           pn.Param(self, parameters=['gen_struct'],
+                                                    show_name=False,
+                                                    widgets={'gen_struct': {'button_type': 'primary'}}),
+                                           pn.Param(self, parameters=['gen_btn'],
+                                                    show_name=False, widgets={'gen_btn': {'button_type': 'primary'}}),
+                                           sizing_mode='stretch_width', margin=(50, 0, 0, 0)
+                                       ),
+                                       self.structure)),
+            ('Preprocess Data', self.rename_files),
             ('View Results', ViewResults().view()),
             ('User Guide', UserGuide().view()),
             dynamic=True, active=0
@@ -212,20 +173,19 @@ class MainArea(param.Parameterized):
             '## Settings',
             '#### Provide output path',
             self.output_path,
-            # '#### Provide short description (max 30 chars)',
-            # self.desc,
-            # '#### Select additional settings',
-            # self.checkbox_group,
-            # # '#### Specify simulation information',
-            # self.speed, self.global_coupling
+            '#### Provide short description (max 30 chars)',
+            self.desc,
+            '#### Select additional settings',
+            self.checkbox_group,
         )
 
         return pn.template.MaterialTemplate(
             title='Preprocess | Transform | Download',
-            # sidebar=sidebar,
-            site='sim2bids',
+            sidebar=sidebar,
+            site='INCF',
             main=main,
-            header_background='#4488c4'
+            header_background='#4488c4',
+
         )
 
 
@@ -292,14 +252,12 @@ class ViewResults(param.Parameterized):
                     name = inp.name.split(' ')[1]
                     content[name] = inp.value
 
-            update_files(content)
-
             # save values
             with open(self.file, 'w') as f:
                 json.dump(content, f)
 
             # update layout
-            je = pn.widgets.JSONEditor(value=json.load(open(self.file)), height=350, mode='view')
+            je = pn.widgets.JSONEditor(value=json.load(self.file), height=350, mode='view')
             je_widget = utils.get_settings(OrderedDict(je.value), self.select_options.value)
             self.widget[-1] = pn.Row(je, pn.Column(je_widget, pn.Param(self, parameters=['je_btn'], show_name=False,
                                                                        widgets={'je_btn': {'button_type': 'primary'}})))
@@ -319,7 +277,9 @@ class UserGuide(param.Parameterized):
         super().__init__(**params)
         self.map = {OPTIONS[0]: ug.how_to_use,
                     OPTIONS[1]: ug.preprocess,
-                    OPTIONS[-1]: ug.bep034}
+                    OPTIONS[2]: ug.supported,
+                    OPTIONS[3]: ug.functionality,
+                    OPTIONS[4]: ug.bep034}
         self.text = pn.widgets.StaticText(value=self.map[OPTIONS[0]])
 
     @pn.depends('user_guide.value', watch=True)
@@ -327,30 +287,7 @@ class UserGuide(param.Parameterized):
         self.text.value = self.map[self.user_guide.value]
 
     def view(self):
-        return pn.Column(self.user_guide, self.text, scroll=True, height=800)
-
-
-def update_files(content):
-    to_ignore = [*templates.required, 'Units', 'ModelEq', 'ModelParam']
-
-    for file in get_files():
-        # get keys for the existing json file in the output folder
-        file_json = json.load(open(file))
-
-        # get the folder name to get the 'recommended' fields
-        folder = os.path.basename(os.path.dirname(file))
-        keys = templates.struct.get(folder, None)
-
-        # filter out only json files
-        if keys:
-            for k, v in content.items():
-                if (k in keys['required'] or k in keys['recommend']) and content[k] and \
-                        (file_json[k] is None or file_json[k] != content[k]) and k not in to_ignore:
-                    file_json[k] = content[k]
-
-        # save new fields
-        with open(file, 'w') as f:
-            json.dump(file_json, f)
+        return pn.Column(self.user_guide, self.text, scroll=True, height=600)
 
 
 def get_files(path=app.OUTPUT, ftype='.json'):
@@ -369,13 +306,9 @@ Here you can select the folder(s) that you want to transform. Beware that we are
 content available in the specified folder. That means if the folder contains a sub-folder, we will transform the content
 if it falls into the accepted file formats.
 
-Click on **Generate Files** button to physically save the results. After that, please 
+You can get a preliminary look into the to-be-generated structure by clicking on **Show Structure** button. If you're 
+happy with the results, you can click on **Generate Files** button to physically save the results. After that, please 
 **make sure to give user-specific input** by clicking on **View Results** tab and selecting JSON files. You'll see a lot
 of required and recommended fields that can be updated. By default, all required fields are already provided. However,
 you have all the rights to change them. See further details in **User Guide**. 
 """
-
-PREPROCESS = """
-### Preprocessing pipeline
-You will see the file names only if there are files that don't match the existing ones. 
-"""
\ No newline at end of file
